<?php
/*
Plugin Name: Amazing Carousel
Plugin URI: http://amazingcarousel.com
Description: Amazing Carousel WordPress Plugin
Version: 3.9
Author: Magic Hills Pty Ltd
Author URI: http://amazingcarousel.com
License: Copyright 2013 Magic Hills Pty Ltd, All Rights Reserved
*/

define('AMAZINGCAROUSEL_VERSION', '3.9');

define('AMAZINGCAROUSEL_URL', plugin_dir_url( __FILE__ ));

define('AMAZINGCAROUSEL_PATH', plugin_dir_path( __FILE__ ));

require_once 'app/class-amazingcarousel-controller.php';

class AmazingCarousel_Plugin

{
	public $amazingcarousel_controller;
	
	function __construct() {
		
		$this->init();
	}
	
	public function init()
	{
		
		$this->amazingcarousel_controller = new AmazingCarousel_Controller();

		register_activation_hook( __FILE__, array($this, 'activation_handler') );
		
		add_action( 'admin_menu', array($this, 'register_menu') );
		
		add_shortcode( 'amazingcarousel', array($this, 'shortcode_handler') );
		
		add_action( 'init', array($this, 'register_script') );
		add_action( 'wp_enqueue_scripts', array($this, 'enqueue_script') );
		add_action( 'admin_enqueue_scripts', array($this, 'enqueue_admin_script') );
	}
	
	function activation_handler()
	{
		$this->amazingcarousel_controller->activation_handler();
	}
	
	function enqueue_admin_script($hook)

	{
		if ($hook == 'admin_page_amazingcarousel_view_carousel')

			wp_enqueue_script('amazingcarousel-script');

	}
	
	function enqueue_script()
	{
		wp_enqueue_script('amazingcarousel-script');
	}
	
	function register_script()
	{
		$uploads = wp_upload_dir();
		$script_url = $uploads['baseurl'] . '/amazingcarousel/sharedengine/amazingcarousel.js';
		
		wp_register_script('amazingcarousel-script', $script_url, array('jquery'), AMAZINGCAROUSEL_VERSION, false);
		
		if ( is_admin() )

		{

			wp_register_style('amazingcarousel-style', AMAZINGCAROUSEL_URL . 'amazingcarousel.css');

			wp_enqueue_style('amazingcarousel-style');

		}
	}
	
	function shortcode_handler($atts)
	{
		if ( !isset($atts['id']) )
			return __('Please specify a carousel id', 'amazingcarousel');
		
		$id = $atts['id'];
		return $this->amazingcarousel_controller->generate_body_code($id);
	}
	
	function register_menu()
	{
		add_menu_page( 
				__('Amazing Carousel', 'amazingcarousel'), 
				__('Amazing Carousel', 'amazingcarousel'), 
				'manage_options', 
				'amazingcarousel_show_carousels', 
				array($this, 'show_carousels'),
				AMAZINGCAROUSEL_URL . 'images/logo-16.png' );
				
		add_submenu_page(
				'amazingcarousel_show_carousels', 
				__('Installed Carousels', 'amazingcarousel'), 
				__('Installed Carousels', 'amazingcarousel'), 
				'manage_options',
				'amazingcarousel_show_carousels',
				array($this, 'show_carousels' ) );
		
		add_submenu_page(

				'amazingcarousel_show_carousels',

				__('Add New carousel', 'amazingcarousel'),

				__('Add New', 'amazingcarousel'),

				'manage_options',

				'amazingcarousel_add_new',

				array($this, 'add_new' ) );
		
		add_submenu_page(

				null,

				__('View Carousel', 'amazingcarousel'),

				__('View Carousel', 'amazingcarousel'),

				'manage_options',

				'amazingcarousel_view_carousel',

				array($this, 'view_carousel' ) );
	}
	
	public function show_carousels()
	{
		$this->amazingcarousel_controller->show_carousels();
	}
	
	public function add_new()
	{
		$this->amazingcarousel_controller->add_new();
	}
	
	public function view_carousel()
	{
		$this->amazingcarousel_controller->view_carousel();
	}
}

$amazingcarousel_plugin = new AmazingCarousel_Plugin();

/**
 * Global php function
 * @param unknown_type $id
 */
function amazingcarousel($id) {
	
	echo $amazingcarousel_plugin->amazingcarousel_controller->generate_body_code($id);
}

/**
 * Uninstallation
 */
function amazingcarousel_uninstall() {


	global $wpdb;
	

	$table_name = $wpdb->prefix . "amazingcarousel";

	$wpdb->query("DROP TABLE IF EXISTS $table_name");

}

if ( function_exists('register_uninstall_hook') )
	register_uninstall_hook( __FILE__, 'amazingcarousel_uninstall' );

