<?php 

require_once 'functions.php';

class AmazingCarousel_Model {

	private $controller;
	
	function __construct($controller) {
		
		$this->controller = $controller;
		
	}
	
	function get_upload_path() {
		
		$uploads = wp_upload_dir();
		
		return $uploads['basedir'] . '/amazingcarousel/';
	}
	
	function get_upload_url() {
	
		$uploads = wp_upload_dir();
	
		return $uploads['baseurl'] . '/amazingcarousel/';
	}
	
	function install_carousel_from_folder($install_folder, $id) {
				
		// write to db
		$ret = $this->add_carousel_to_db($install_folder . 'data/', $id);
		if ( is_wp_error($ret) )
			return $ret;
		
		// copy to uploads folder
		$upload_folder = $this->get_upload_path();
		if ( !is_dir( $upload_folder ) )
		{
			if ( 0 == wp_mkdir_p( $upload_folder ) )
				return new WP_Error('filesystem', __("Cannot create folder", "amazingcarousel"));
		}
		
		$carousel_folder = $upload_folder . $ret['id'] . '/';
		if ( is_dir( $carousel_folder ) )
			amazingcarousel_recurse_rmdir( $carousel_folder );
		if ( 0 == wp_mkdir_p( $carousel_folder ) )
			return new WP_Error('filesystem', __("Cannot create folder", "amazingcarousel"));
		
		amazingcarousel_recurse_copy($install_folder . 'data/', $carousel_folder);
		
		// copy sharedengine to upload folder
		$sharedengine_folder = $upload_folder . 'sharedengine/';
		if ( is_dir( $sharedengine_folder ) )
			amazingcarousel_recurse_rmdir( $sharedengine_folder );
		if ( 0 == wp_mkdir_p( $sharedengine_folder ) )
			return new WP_Error('filesystem', __("Cannot create folder", "amazingcarousel"));
		
		amazingcarousel_recurse_copy($install_folder . 'sharedengine/', $sharedengine_folder);
		
		// modify id of carousel
		$file = $carousel_folder . 'carouselengine/initcarousel.js';
		$content = file_get_contents($file);
		$content = str_replace("%CAROUSELID%", $ret['id'], $content);
		$jsfolder = $this->get_upload_url() . $ret['id'] . '/carouselengine/';
		$content = str_replace("%JSFOLDER%", $jsfolder, $content);
		file_put_contents($file, $content);

		// modify id in css
		$file = $carousel_folder . 'carouselengine/initcarousel.css';
		$content = file_get_contents($file);
		$content = str_replace("%CAROUSELID%", $ret['id'], $content);
		file_put_contents($file, $content);
		
		return array(
				"id" => $ret['id']);
	}
	
	function install_carousel($zipfile, $id) {
				
		// unzip
		$unzip_folder = trailingslashit ( dirname( $zipfile ) );
		
		$ret = amazingcarousel_unzip_file($zipfile, $unzip_folder);
		if ( is_wp_error($ret) )
			return $ret;
		
		unlink( $zipfile );
		
		$ret = $this->install_carousel_from_folder( $unzip_folder . "amazingcarousel-plugin/", $id );
		if ( is_wp_error($ret) )
			return $ret;
		
		// check new version
		$ret["new"] = false;
		
		$file = $unzip_folder . "amazingcarousel-plugin/" . "amazingcarousel.php";
		$content = file_get_contents($file);
		$pattern = '/define\(\'AMAZINGCAROUSEL_VERSION\', \'([0-9\.]+)\'\);/';
		if ( preg_match($pattern, $content, $matches) )
		{
			if ( floatval($matches[1]) > floatval(AMAZINGCAROUSEL_VERSION) )
				$ret["new"] = true;
		}
		
		amazingcarousel_recurse_rmdir( $unzip_folder . "amazingcarousel-plugin/" );
		
		return $ret;
	}
	
	
	function add_carousel_to_db($folder, $id) {
		
		$this->create_db_table();
		return $this->insert_carousel_to_db_table($folder, $id);
	}
	
	function is_db_table_exists() {
		
		global $wpdb;
		$table_name = $wpdb->prefix . "amazingcarousel";
		
		return ( $wpdb->get_var("SHOW TABLES LIKE '$table_name'") == $table_name );
	}
	
	function create_db_table() {
		
		global $wpdb;
		$table_name = $wpdb->prefix . "amazingcarousel";
		
		$charset = '';
		if ( !empty($wpdb -> charset) ) 
			$charset = "DEFAULT CHARACTER SET $wpdb->charset";
		if ( !empty($wpdb -> collate) ) 
			$charset .= " COLLATE $wpdb->collate";
		
		$sql = "CREATE TABLE $table_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
		authorid tinytext NOT NULL,
		name tinytext DEFAULT '' NOT NULL,
		images text DEFAULT '' NOT NULL,
		carouselcode text DEFAULT '' NOT NULL,
		PRIMARY KEY  (id)
		) $charset;";
			
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
	}
	
	function insert_carousel_to_db_table($folder, $id) {
		
		global $wpdb, $user_ID;
		$table_name = $wpdb->prefix . "amazingcarousel";
		
		$file = $folder . "carousel.html";
		if ( !is_file($file) )
			return new WP_Error('filesystem', __("Cannot find carousel file", "amazingcarousel"));
		
		$time = current_time('mysql');
		$authorid = $user_ID;
		
		if ( ($id > 0) && $this->is_id_exist($id) )
		{
			$ret = $wpdb->query( $wpdb->prepare(
					"
					UPDATE $table_name
					SET time=%s, authorid=%s 
					WHERE id=%d
					",
					$time,
					$authorid,
					$id
			) );	
			if (!$ret)
				return new WP_Error('wp_db', __("Cannot insert carousel to database", "amazingcarousel"));
		}
		else
		{
			$ret = $wpdb->query( $wpdb->prepare(
					"
					INSERT INTO $table_name (time, authorid, name, images, carouselcode)
					VALUES (%s, %s, %s, %s, %s)
					",
					$time,
					$authorid,
					"",
					"",
					""
			) );
			if (!$ret)
				return new WP_Error('wp_db', __("Cannot insert carousel to database", "amazingcarousel"));
			
			$id = $wpdb->insert_id;
		}
		
		$ret = $this->read_carousel($file, $id);
		if ( is_wp_error($ret) )
			return $ret;
		
		$name = $ret['name'];
		$images = $ret['images'];
		$carouselcode = $ret['carouselcode'];
		
		$ret = $wpdb->query( $wpdb->prepare(
				"
				UPDATE $table_name 
				SET name=%s, images=%s, carouselcode=%s 
				WHERE id=%d
				",
				$name,
				$images,
				$carouselcode,
				$id
		) );
		
		return array(
				"id" => $id);
	}
	
	function read_carousel($file, $id) {
		
		$dest_url = $this->get_upload_url() . $id . '/';
		$content = file_get_contents($file);
		
		$name = "";
		$pattern = '/<title>(.*)<\/title>/';
		if ( preg_match($pattern, $content, $matches) )
			$name = $matches[1];
						
		$carouselcode = "";
		$pattern = '/<!-- Insert to your webpage where you want to display the carousel -->(.*)<!-- End of body section HTML codes -->/s';
		if ( preg_match($pattern, $content, $matches) )
		{
			$carouselcode = str_replace("%DESTURL%", $dest_url, $matches[1]);
			$carouselcode = str_replace("%CAROUSELID%", $id, $carouselcode);
		}
		
		$imagescode = $carouselcode;
		$pattern = '/<ul class=[\"\']amazingcarousel-slides[\"\'].*<\/ul>/sU';
		if ( preg_match($pattern, $carouselcode, $matches) )
		{
			$imagescode = $matches[0];
		}
		
		$images = "";
		$pattern = '/<img (data-)*src="([^"]+)"/';
		if ( preg_match_all($pattern, $imagescode, $matches) )
			$images = serialize($matches[2]);
	
		return array(
				"name" => $name,
				"images" => $images,
				"carouselcode" => $carouselcode);
	}
	
	function is_id_exist($id)
	{
		global $wpdb;
		$table_name = $wpdb->prefix . "amazingcarousel";
		
		$carousel_row = $wpdb->get_row( $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $id) );
		return ($carousel_row != null);
	}
	
	function generate_body_code($id)
	{
		global $wpdb;
		$table_name = $wpdb->prefix . "amazingcarousel";
		
		$ret = "";
		$carousel_row = $wpdb->get_row( $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $id) );
		if ($carousel_row != null)
		{
			$ret = '<link rel="stylesheet" type="text/css" media="all" href="' . $this->get_upload_url() . $id . '/carouselengine/initcarousel.css" />' . "\n";
			$ret .= $carousel_row->carouselcode;
			$ret .= '<script src="' . $this->get_upload_url() . $id . '/carouselengine/initcarousel.js"></script>' . "\n";
		}
		else 
		{
			$ret = '<p>The specified carousel id does not exist.</p>';
		}
		return $ret;
	}
	
	function get_list_data()
	{
		
		global $wpdb;
		$table_name = $wpdb->prefix . "amazingcarousel";
		
		$carousel_rows = $wpdb->get_results( "SELECT * FROM $table_name", ARRAY_A);
		
		$ret = array();
		
		if ( $carousel_rows )
		{
			foreach ( $carousel_rows as $row )
			{
				$ret[] = array(
							"id" => $row['id'],
							'name' => $row['name'],
							'images' => $row['images'],
							'time' => $row['time'],
							'author' => $row['authorid']
						);
			}
		}
		
		return $ret;
	}
	
	function delete_carousel($id)
	{
		global $wpdb;
		$table_name = $wpdb->prefix . "amazingcarousel";
		
		$ret = $wpdb->query( $wpdb->prepare(
				"
				DELETE FROM $table_name WHERE id=%s
				",
				$id
		) );
		
		return $ret;
	}
}